'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = defaultChunkParser;
var entryDelimiter = '\n';

// The defaultChunkParser expects the response from the server to consist of new-line
// delimited JSON, eg:
//
//  { "chunk": "#1", "data": "Hello" }
//  { "chunk": "#2", "data": "World" }
//
// It will correctly handle the case where a chunk is emitted by the server across
// delimiter boundaries.
function defaultChunkParser(rawChunk) {
  var prevChunkSuffix = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

  var chunkSuffix = void 0;

  var rawChunks = ('' + prevChunkSuffix + rawChunk).split(entryDelimiter);

  if (!hasSuffix(rawChunk, entryDelimiter)) {
    chunkSuffix = rawChunks.pop();
  }

  var processedChunks = rawChunks.filter(function (v) {
    return v.trim() !== '';
  }).map(function (v) {
    return JSON.parse(v);
  });

  return [processedChunks, chunkSuffix];
}

function hasSuffix(s, suffix) {
  return s.substr(s.length - suffix.length) === suffix;
}