'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = mozXhrRequest;
var MOZ_CHUNKED = exports.MOZ_CHUNKED = 'moz-chunked';

function mozXhrRequest(options) {
  var xhr = new XMLHttpRequest();

  function onProgressEvent() {
    var view = new Uint8Array(xhr.response);
    var len = view.length;

    var rawString = new Array(len);
    while (len--) {
      rawString[len] = String.fromCharCode(view[len]);
    }
    options.onRawChunk(rawString.join(''));
  }

  function onLoadEvent() {
    options.onComplete({
      statusCode: xhr.status,
      transport: MOZ_CHUNKED,
      raw: xhr
    });
  }

  xhr.open(options.method, options.url);
  xhr.responseType = 'moz-chunked-arraybuffer';
  if (options.headers) {
    Object.getOwnPropertyNames(options.headers).forEach(function (k) {
      xhr.setRequestHeader(k, options.headers[k]);
    });
  }
  if (options.credentials === 'include') {
    xhr.withCredentials = true;
  }
  xhr.addEventListener('progress', onProgressEvent);
  xhr.addEventListener('load', onLoadEvent);
  xhr.send(options.body);
}